@@ -25,10 +25,11 @@
 #include <ktcpsocket.h>
 
 #include "job_p.h"
-#include "message_p.h"
+#include "response_p.h"
 #include "session_p.h"
 #include "rfccodecs.h"
 
+
 #include "common.h"
 
 extern "C" {
@@ -36,14 +37,14 @@ extern "C" {
 }
 
 static const sasl_callback_t callbacks[] = {
-    { SASL_CB_ECHOPROMPT, Q_NULLPTR, nullptr },
-    { SASL_CB_NOECHOPROMPT, Q_NULLPTR, nullptr },
-    { SASL_CB_GETREALM, Q_NULLPTR, nullptr },
-    { SASL_CB_USER, Q_NULLPTR, nullptr },
-    { SASL_CB_AUTHNAME, Q_NULLPTR, nullptr },
-    { SASL_CB_PASS, Q_NULLPTR, nullptr },
-    { SASL_CB_CANON_USER, Q_NULLPTR, nullptr },
-    { SASL_CB_LIST_END, Q_NULLPTR, nullptr }
+    { SASL_CB_ECHOPROMPT, nullptr, nullptr },
+    { SASL_CB_NOECHOPROMPT, nullptr, nullptr },
+    { SASL_CB_GETREALM, nullptr, nullptr },
+    { SASL_CB_USER, nullptr, nullptr },
+    { SASL_CB_AUTHNAME, nullptr, nullptr },
+    { SASL_CB_PASS, nullptr, nullptr },
+    { SASL_CB_CANON_USER, nullptr, nullptr },
+    { SASL_CB_LIST_END, nullptr, nullptr }
 };
 
 namespace KIMAP
@@ -60,8 +61,8 @@ public:
 
     LoginJobPrivate(LoginJob *job, Session *session, const QString &name) : JobPrivate(session, name), q(job), encryptionMode(LoginJob::Unencrypted), authState(Login), plainLoginDisabled(false)
     {
-        conn = Q_NULLPTR;
-        client_interact = Q_NULLPTR;
+        conn = nullptr;
+        client_interact = nullptr;
     }
     ~LoginJobPrivate() { }
     bool sasl_interact();
@@ -69,7 +70,7 @@ public:
     bool startAuthentication();
     bool answerChallenge(const QByteArray &data);
     void sslResponse(bool response);
-    void saveServerGreeting(const Message &response);
+    void saveServerGreeting(const Response &response);
 
     LoginJob *q;
 
@@ -113,22 +114,22 @@ bool LoginJobPrivate::sasl_interact()
         case SASL_CB_AUTHNAME:
             if (!authorizationName.isEmpty()) {
                 qCDebug(KIMAP_LOG) << "SASL_CB_[AUTHNAME]: '" << authorizationName << "'";
-                interact->result = strdup(authorizationName.toUtf8());
+                interact->result = strdup(authorizationName.toUtf8().constData());
                 interact->len = strlen((const char *) interact->result);
                 break;
             }
         case SASL_CB_USER:
             qCDebug(KIMAP_LOG) << "SASL_CB_[USER|AUTHNAME]: '" << userName << "'";
-            interact->result = strdup(userName.toUtf8());
+            interact->result = strdup(userName.toUtf8().constData());
             interact->len = strlen((const char *) interact->result);
             break;
         case SASL_CB_PASS:
             qCDebug(KIMAP_LOG) << "SASL_CB_PASS: [hidden]";
-            interact->result = strdup(password.toUtf8());
+            interact->result = strdup(password.toUtf8().constData());
             interact->len = strlen((const char *) interact->result);
             break;
         default:
-            interact->result = Q_NULLPTR;
+            interact->result = nullptr;
             interact->len = 0;
             break;
         }
@@ -268,7 +269,7 @@ void LoginJob::doStart()
     }
 }
 
-void LoginJob::handleResponse(const Message &response)
+void LoginJob::handleResponse(const Response &response)
 {
     Q_D(LoginJob);
 
@@ -334,7 +335,7 @@ void LoginJob::handleResponse(const Message &response)
     case UNTAGGED:
         // The only untagged response interesting for us here is CAPABILITY
         if (response.content[1].toString() == "CAPABILITY") {
-            QList<Message::Part>::const_iterator p = response.content.begin() + 2;
+            QList<Response::Part>::const_iterator p = response.content.begin() + 2;
             while (p != response.content.end()) {
                 QString capability = QLatin1String(p->toString());
                 d->capabilities << capability;
@@ -403,9 +404,9 @@ void LoginJob::handleResponse(const Message &response)
             } else {
                 bool authModeSupported = false;
                 //find the selected SASL authentication method
-                Q_FOREACH (const QString &capability, d->capabilities) {
+                for (const QString &capability : qAsConst(d->capabilities)) {
                     if (capability.startsWith(QLatin1String("AUTH="))) {
-                        if (capability.mid(5) == d->authMode) {
+                        if (capability.midRef(5) == d->authMode) {
                             authModeSupported = true;
                             break;
                         }
@@ -424,6 +425,9 @@ void LoginJob::handleResponse(const Message &response)
         case LoginJobPrivate::Authenticate:
             sasl_dispose(&d->conn);   //SASL authentication done
         // Fall through
+#if QT_VERSION >= QT_VERSION_CHECK(5,8,0)
+        Q_FALLTHROUGH();
+#endif
         case LoginJobPrivate::Login:
             d->saveServerGreeting(response);
             emitResult(); //got an OK, command done
@@ -449,20 +453,22 @@ bool LoginJobPrivate::startAuthentication()
     }
 
     authState = LoginJobPrivate::Authenticate;
-    const char *out = Q_NULLPTR;
+    const char *out = nullptr;
     uint outlen = 0;
-    const char *mechusing = Q_NULLPTR;
+    const char *mechusing = nullptr;
 
-    int result = sasl_client_new("imap", m_session->hostName().toLatin1(), Q_NULLPTR, nullptr, callbacks, 0, &conn);
+    int result = sasl_client_new("imap", m_session->hostName().toLatin1().constData(), nullptr, nullptr, callbacks, 0, &conn);
     if (result != SASL_OK) {
-        qCDebug(KIMAP_LOG) << "sasl_client_new failed with:" << result;
+        const QString saslError = QString::fromUtf8(sasl_errdetail(conn));
+        qCWarning(KIMAP_LOG) << "sasl_client_new failed with:" << result << saslError;
         q->setError(LoginJob::UserDefinedError);
-        q->setErrorText(QString::fromUtf8(sasl_errdetail(conn)));
+        q->setErrorText(saslError);
         return false;
     }
 
     do {
-        result = sasl_client_start(conn, authMode.toLatin1(), &client_interact, capabilities.contains(QStringLiteral("SASL-IR")) ? &out : Q_NULLPTR, &outlen, &mechusing);
+        qCDebug(KIMAP_LOG) << "Trying authmod" << authMode.toLatin1();
+        result = sasl_client_start(conn, authMode.toLatin1().constData(), &client_interact, capabilities.contains(QStringLiteral("SASL-IR")) ? &out : nullptr, &outlen, &mechusing);
 
         if (result == SASL_INTERACT) {
             if (!sasl_interact()) {
@@ -474,9 +480,10 @@ bool LoginJobPrivate::startAuthentication()
     } while (result == SASL_INTERACT);
 
     if (result != SASL_CONTINUE && result != SASL_OK) {
-        qCDebug(KIMAP_LOG) << "sasl_client_start failed with:" << result;
+        const QString saslError = QString::fromUtf8(sasl_errdetail(conn));
+        qCWarning(KIMAP_LOG) << "sasl_client_start failed with:" << result << saslError;
         q->setError(LoginJob::UserDefinedError);
-        q->setErrorText(QString::fromUtf8(sasl_errdetail(conn)));
+        q->setErrorText(saslError);
         sasl_dispose(&conn);
         return false;
     }
@@ -497,10 +504,10 @@ bool LoginJobPrivate::answerChallenge(const QByteArray &data)
 {
     QByteArray challenge = data;
     int result = -1;
-    const char *out = Q_NULLPTR;
+    const char *out = nullptr;
     uint outlen = 0;
     do {
-        result = sasl_client_step(conn, challenge.isEmpty() ? Q_NULLPTR : challenge.data(),
+        result = sasl_client_step(conn, challenge.isEmpty() ? nullptr : challenge.data(),
                                   challenge.size(),
                                   &client_interact,
                                   &out, &outlen);
@@ -515,9 +522,10 @@ bool LoginJobPrivate::answerChallenge(const QByteArray &data)
     } while (result == SASL_INTERACT);
 
     if (result != SASL_CONTINUE && result != SASL_OK) {
-        qCDebug(KIMAP_LOG) << "sasl_client_step failed with:" << result;
+        const QString saslError = QString::fromUtf8(sasl_errdetail(conn));
+        qCWarning(KIMAP_LOG) << "sasl_client_step failed with:" << result << saslError;
         q->setError(LoginJob::UserDefinedError);   //TODO: check up the actual error
-        q->setErrorText(QString::fromUtf8(sasl_errdetail(conn)));
+        q->setErrorText(saslError);
         sasl_dispose(&conn);
         return false;
     }
@@ -601,15 +609,16 @@ void LoginJob::connectionLost()
 
 }
 
-void LoginJobPrivate::saveServerGreeting(const Message &response)
+void LoginJobPrivate::saveServerGreeting(const Response &response)
 {
     // Concatenate the parts of the server response into a string, while dropping the first two parts
     // (the response tag and the "OK" code), and being careful not to add useless extra whitespace.
 
     for (int i = 2; i < response.content.size(); i++) {
-        if (response.content.at(i).type() == Message::Part::List) {
+        if (response.content.at(i).type() == Response::Part::List) {
             serverGreeting += QLatin1Char('(');
-            foreach (const QByteArray &item, response.content.at(i).toList()) {
+            const QList<QByteArray> itemLst = response.content.at(i).toList();
+            for (const QByteArray &item : itemLst) {
                 serverGreeting += QLatin1String(item) + QLatin1Char(' ');
             }
             serverGreeting.chop(1);
